/*
 * Copyright 2007 Geir Hedemark
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.testaco.test;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;


import org.springframework.test.AbstractDependencyInjectionSpringContextTests;
import org.testaco.test.annotations.CheckDb;
import org.testaco.test.annotations.LoadDb;

/**
 * TODO docs of class
 * @author geir
 *
 */
public abstract class SpringIntegrationTestCase extends
		AbstractDependencyInjectionSpringContextTests {
	
	/**
	 * Please override the onOnSetUp method to introduce setup code.
	 */
	@Override
	protected final void onSetUp() throws Exception {
		onOnSetUp();
		checkSetupAnnotations();
		super.onSetUp();
	}
	protected void onOnSetUp() {
		
	}
	/**
	 * Please override the onOnTearDown method to introduce tear down code.
	 */
	@Override
	protected final void onTearDown() throws Exception {
		onOnTearDown();
		checkTearDownAnnotations();
		super.onTearDown();
	}
	protected void onOnTearDown() {
		
	}
	
	/**
	 * TODO docs of setup annotations
	 * @throws Exception
	 */
	private void checkSetupAnnotations() throws Exception {
		//Unit under test
		String uut = this.getName(); 
		/*The "getName" method returns the name of the 
		 * method that is currently being tested in 
		 * junit 3.8.x. I hope that doesn't change. */
		Method uutMethod= null;
		try {
			uutMethod= getClass().getMethod(uut, (Class[])null);
		} catch (NoSuchMethodException e) {
			fail("Method \""+uut+"\" not found");
		}
		Annotation[] testAnnotations = uutMethod.getAnnotations();
		for (int i = 0; i < testAnnotations.length; i++) {
			Annotation annotation = testAnnotations[i];
			if (annotation instanceof LoadDb) {
				LoadDb loadDbAnnotation = (LoadDb) annotation;
				String databaseKey = loadDbAnnotation.database();
				String dataFileName = loadDbAnnotation.datafile();
				DataSetHelper.loadDataSet(databaseKey, dataFileName);
			}
		}
	}

	/**
	 * TODO docs of annotations
	 * @throws Exception
	 */
	private void checkTearDownAnnotations() throws Exception {
		String uut = this.getName();
		Method uutMethod= null;
		try {
			uutMethod= getClass().getMethod(uut, (Class[])null);
		} catch (NoSuchMethodException e) {
			fail("Method \""+uut+"\" not found");
		}
		Annotation[] testAnnotations = uutMethod.getAnnotations();
		for (int i = 0; i < testAnnotations.length; i++) {
			Annotation annotation = testAnnotations[i];
			if (annotation instanceof CheckDb) {
				CheckDb checkDbAnnotation = (CheckDb) annotation;
				String databaseKey = checkDbAnnotation.database();
				String dataFileName = checkDbAnnotation.datafile();
				DataSetHelper.checkDataset(databaseKey, dataFileName);
			}
		}
	}
}
